#include "../vecmap.h"

//#define USE_VECQUE
#ifdef USE_VECQUE
#include "../../mem1_allocator/mem1_allocator.h"
#include "../../vecque/vecque.h"
#include "../../ya_vector/ya_vector.h"
#include "../../ya_string/ya_string.h"
#define string 	ya_string
#define vector	ya_vector
#else
#include <vector>
#include <string>
#endif
#include "unitst.h"

using namespace std;


UNITST_TEST(vecmap_test) {
  #ifdef USE_VECQUE
	//typedef vecmap<std::string, int, vecque< std::pair<std::string,int> > >	vm_t;
	typedef vecmap<string
					, int
					, std::less< string >
					, vecque< std::pair<string,int>
							, static_mem1_allocator<std::pair<string,int>, 0x8000>
							//, std::allocator< std::pair<string,int> >
							// , std::vector< std::pair<string,int>* , std::allocator< std::pair<string,int>* > >
							, vector< std::pair<string,int>*, ya_allocator< std::pair<string,int>* > >
							>
				  >	vm_t;
  #else
	typedef vecmap<string, int>	vm_t;
  #endif
	vm_t		vm;
	vm.reserve( 4 );
	UNITST_EQ(vm.empty(), 1);

	vm[ string("123") ] = 123;
	vm[ string("789") ] = 789;
	vm[ string("456") ] = 456;

	UNITST_EQ(vm.empty(), 0);
	UNITST_EQ(vm[string("123")], 123);
	UNITST_EQ(vm[string("456")], 456);
	UNITST_EQ(vm[string("789")], 789);

	UNITST_EQ(vm.size(), 3);
	UNITST_GE(vm.capacity(), 4);
	UNITST_GE(vm.max_size(), 4);

	UNITST_EQ(vm.count(string("456")), 1);
	UNITST_EQ(vm.count(string("ABC")), 0);
	UNITST_CHECK(vm.find(string("123")) != vm.end());
	UNITST_CHECK(vm.find(string("ABC")) == vm.end());

	unsigned i = 0;
	for (vm_t::iterator it	= vm.begin();
						it != vm.end();
						++it)
	{
		static const char* s_str[] = { "123","456","789" };
		UNITST_CHECK( it->first == string(s_str[i++]) );
	}
	vm.erase("456");
	UNITST_EQ(vm.size(), 2);
	vm.erase("123");
	UNITST_EQ(vm.size(), 1);
	vm.clear();
	UNITST_EQ(vm.size(), 0);
	UNITST_EQ(vm.empty(), 1);

	vm.insert(vm_t::pair_type(string("1"),1));
	// std::pair<string,int> tbl2[] = { {string("2"),2},{string("3"),3},{string("4"),4},{string("5"),5} };
	vector< vm_t::pair_type > tbl2;
	tbl2.push_back( vm_t::pair_type( string("3"),3 ) );
	tbl2.push_back( vm_t::pair_type( string("5"),5 ) );
	tbl2.push_back( vm_t::pair_type( string("2"),2 ) );
	tbl2.push_back( vm_t::pair_type( string("4"),4 ) );
	vm.insert(&tbl2[0], &tbl2[3]+1);
	vm.insert(vm.begin()+3, vm_t::pair_type(string("6"),6) );

	UNITST_EQ(vm[string("1")], 1);
	UNITST_EQ(vm[string("2")], 2);
	UNITST_EQ(vm[string("3")], 3);
	UNITST_EQ(vm[string("4")], 4);
	UNITST_EQ(vm[string("5")], 5);
	UNITST_EQ(vm[string("6")], 6);

	vm_t	vm2 = vm;
	UNITST_EQ(vm2, vm);
	vm_t	vm3;
	using std::swap;
	swap(vm3, vm2);
	UNITST_EQ(vm3, vm);
	UNITST_EQ(vm == vm3, 1);
	UNITST_EQ(vm >= vm3, 1);
	UNITST_EQ(vm <= vm3, 1);
	UNITST_EQ(vm != vm3, 0);
	UNITST_EQ(vm <	vm3, 0);
	UNITST_EQ(vm >	vm3, 0);

	UNITST_EQ(vm == vm2, 0);
	UNITST_EQ(vm >= vm2, 1);
	UNITST_EQ(vm <= vm2, 0);
	UNITST_EQ(vm != vm2, 1);
	UNITST_EQ(vm <	vm2, 0);
	UNITST_EQ(vm >	vm2, 1);

	UNITST_EQ(vm2 == vm3, 0);
	UNITST_EQ(vm2 >= vm3, 0);
	UNITST_EQ(vm2 <= vm3, 1);
	UNITST_EQ(vm2 != vm3, 1);
	UNITST_EQ(vm2 <  vm3, 1);
	UNITST_EQ(vm2 >  vm3, 0);
}

